@@ -77,6 +77,8 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Suunto", "D6i",  DC_FAMILY_SUUNTO_D9, 0x1A},
 	{"Suunto", "D9tx", DC_FAMILY_SUUNTO_D9, 0x1B},
 	{"Suunto", "DX",   DC_FAMILY_SUUNTO_D9, 0x1C},
+	{"Suunto", "Vyper Novo", DC_FAMILY_SUUNTO_D9, 0x1D},
+	{"Suunto", "Zoop Novo",  DC_FAMILY_SUUNTO_D9, 0x1E},
 	/* Suunto EON Steel */
 #ifdef HAVE_LIBUSB
 	{"Suunto", "EON Steel", DC_FAMILY_SUUNTO_EONSTEEL, 0},
@@ -104,6 +106,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Subgear","XP-10",         DC_FAMILY_UWATEC_SMART, 0x13},
 	{"Uwatec", "Smart Com",     DC_FAMILY_UWATEC_SMART, 0x14},
 	{"Uwatec", "Aladin 2G",     DC_FAMILY_UWATEC_SMART, 0x15},
+	{"Uwatec", "Aladin Sport",  DC_FAMILY_UWATEC_SMART, 0x15},
 	{"Subgear","XP-3G",         DC_FAMILY_UWATEC_SMART, 0x15},
 	{"Uwatec", "Smart Tec",     DC_FAMILY_UWATEC_SMART, 0x18},
 	{"Uwatec", "Galileo Trimix",DC_FAMILY_UWATEC_SMART, 0x19},
@@ -181,6 +184,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Aeris",    "A300CS",              DC_FAMILY_OCEANIC_ATOM2, 0x454C},
 	{"Oceanic",  "F11",                 DC_FAMILY_OCEANIC_ATOM2, 0x4554},
 	{"Oceanic",  "VTX",                 DC_FAMILY_OCEANIC_ATOM2, 0x4557},
+	{"Aqualung", "i450T",               DC_FAMILY_OCEANIC_ATOM2, 0x4641},
 	/* Mares Nemo */
 	{"Mares", "Nemo",         DC_FAMILY_MARES_NEMO, 0},
 	{"Mares", "Nemo Steel",   DC_FAMILY_MARES_NEMO, 0},
@@ -216,6 +220,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Heinrichs Weikamp", "OSTC 2",     DC_FAMILY_HW_OSTC3, 0x11},
 	{"Heinrichs Weikamp", "OSTC 3",     DC_FAMILY_HW_OSTC3, 0x0A},
 	{"Heinrichs Weikamp", "OSTC 3",     DC_FAMILY_HW_OSTC3, 0x1A},
+	{"Heinrichs Weikamp", "OSTC 4",     DC_FAMILY_HW_OSTC3, 0x3B},
 	{"Heinrichs Weikamp", "OSTC cR",    DC_FAMILY_HW_OSTC3, 0x05},
 	{"Heinrichs Weikamp", "OSTC Sport", DC_FAMILY_HW_OSTC3, 0x12},
 	/* Cressi Edy */
@@ -224,6 +229,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	/* Cressi Leonardo */
 	{"Cressi", "Leonardo", DC_FAMILY_CRESSI_LEONARDO, 1},
 	{"Cressi", "Giotto",   DC_FAMILY_CRESSI_LEONARDO, 4},
+	{"Cressi", "Newton",   DC_FAMILY_CRESSI_LEONARDO, 5},
 	/* Zeagle N2iTiON3 */
 	{"Zeagle",   "N2iTiON3",   DC_FAMILY_ZEAGLE_N2ITION3, 0},
 	{"Apeks",    "Quantum X",  DC_FAMILY_ZEAGLE_N2ITION3, 0},
@@ -240,6 +246,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Shearwater", "Petrel",   DC_FAMILY_SHEARWATER_PETREL, 3},
 	{"Shearwater", "Petrel 2", DC_FAMILY_SHEARWATER_PETREL, 3},
 	{"Shearwater", "Nerd",     DC_FAMILY_SHEARWATER_PETREL, 3},
+	{"Shearwater", "Perdix",   DC_FAMILY_SHEARWATER_PETREL, 3},
 	/* Dive Rite NiTek Q */
 	{"Dive Rite", "NiTek Q",   DC_FAMILY_DIVERITE_NITEKQ, 0},
 	/* Citizen Hyper Aqualand */
@@ -259,6 +266,10 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"DiveSystem", "iX3M Deep",     DC_FAMILY_DIVESYSTEM_IDIVE, 0x23},
 	{"DiveSystem", "iX3M Tec",      DC_FAMILY_DIVESYSTEM_IDIVE, 0x24},
 	{"DiveSystem", "iX3M Reb",      DC_FAMILY_DIVESYSTEM_IDIVE, 0x25},
+	{"Cochran", "Commander",	DC_FAMILY_COCHRAN_COMMANDER, 0},
+	{"Cochran", "EMC-14",		DC_FAMILY_COCHRAN_COMMANDER, 1},
+	{"Cochran", "EMC-16",		DC_FAMILY_COCHRAN_COMMANDER, 2},
+	{"Cochran", "EMC-20H",		DC_FAMILY_COCHRAN_COMMANDER, 3},
 };
 
 typedef struct dc_descriptor_iterator_t {
