@@ -137,7 +137,7 @@ serial_open (serial_t **out, dc_context_t *context, const char* name)
 	// Allocate memory.
 	serial_t *device = (serial_t *) malloc (sizeof (serial_t));
 	if (device == NULL) {
-		SYSERROR (context, errno);
+		SYSERROR (context, ENOMEM);
 		return -1; // ENOMEM (Not enough space)
 	}
 
@@ -157,17 +157,14 @@ serial_open (serial_t **out, dc_context_t *context, const char* name)
 	device->fd = open (name, O_RDWR | O_NOCTTY | O_NONBLOCK);
 	if (device->fd == -1) {
 		SYSERROR (context, errno);
-		free (device);
-		return -1; // Error during open call.
+		goto error_free;
 	}
 
 #ifndef ENABLE_PTY
 	// Enable exclusive access mode.
 	if (ioctl (device->fd, TIOCEXCL, NULL) != 0) {
 		SYSERROR (context, errno);
-		close (device->fd);
-		free (device);
-		return -1;
+		goto error_close;
 	}
 #endif
 
@@ -177,14 +174,18 @@ serial_open (serial_t **out, dc_context_t *context, const char* name)
 	// file descriptor represents a terminal device.
 	if (tcgetattr (device->fd, &device->tty) != 0) {
 		SYSERROR (context, errno);
-		close (device->fd);
-		free (device);
-		return -1;
+		goto error_close;
 	}
 
 	*out = device;
 
 	return 0;
+
+error_close:
+	close (device->fd);
+error_free:
+	free (device);
+	return -1;
 }
 
 //
@@ -194,15 +195,15 @@ serial_open (serial_t **out, dc_context_t *context, const char* name)
 int
 serial_close (serial_t *device)
 {
+	int errcode = 0;
+
 	if (device == NULL)
 		return 0;
 
 	// Restore the initial terminal attributes.
 	if (tcsetattr (device->fd, TCSANOW, &device->tty) != 0) {
 		SYSERROR (device->context, errno);
-		close (device->fd);
-		free (device);
-		return -1;
+		errcode = -1;
 	}
 
 #ifndef ENABLE_PTY
@@ -213,14 +214,13 @@ serial_close (serial_t *device)
 	// Close the device.
 	if (close (device->fd) != 0) {
 		SYSERROR (device->context, errno);
-		free (device);
-		return -1;
+		errcode = -1;
 	}
 
 	// Free memory.
 	free (device);
 
-	return 0;
+	return errcode;
 }
 
 //
