@@ -150,8 +150,7 @@ serial_open (serial_t **out, dc_context_t *context, const char* name)
 			NULL);
 	if (device->hFile == INVALID_HANDLE_VALUE) {
 		SYSERROR (context, GetLastError ());
-		free (device);
-		return -1;
+		goto error_free;
 	}
 
 	// Retrieve the current communication settings and timeouts,
@@ -161,14 +160,18 @@ serial_open (serial_t **out, dc_context_t *context, const char* name)
 	if (!GetCommState (device->hFile, &device->dcb) ||
 		!GetCommTimeouts (device->hFile, &device->timeouts)) {
 		SYSERROR (context, GetLastError ());
-		CloseHandle (device->hFile);
-		free (device);
-		return -1;
+		goto error_close;
 	}
 
 	*out = device;
 
 	return 0;
+
+error_close:
+	CloseHandle (device->hFile);
+error_free:
+	free (device);
+	return -1;
 }
 
 //
@@ -178,6 +181,8 @@ serial_open (serial_t **out, dc_context_t *context, const char* name)
 int
 serial_close (serial_t *device)
 {
+	int errcode = 0;
+
 	if (device == NULL)
 		return 0;
 
@@ -185,22 +190,19 @@ serial_close (serial_t *device)
 	if (!SetCommState (device->hFile, &device->dcb) ||
 		!SetCommTimeouts (device->hFile, &device->timeouts)) {
 		SYSERROR (device->context, GetLastError ());
-		CloseHandle (device->hFile);
-		free (device);
-		return -1;
+		errcode = -1;
 	}
 
 	// Close the device.
 	if (!CloseHandle (device->hFile)) {
 		SYSERROR (device->context, GetLastError ());
-		free (device);
-		return -1;
+		errcode = -1;
 	}
 
 	// Free memory.
 	free (device);
 
-	return 0;
+	return errcode;
 }
 
 //
